#include "ShaderConstants.fxh"
#include "util.fxh"

struct P_in
{
	float4 position : SV_Position;

#ifndef BYPASS_PIXEL_SHADER
	lpfloat4 color : COLOR;
	snorm float2 uv0 : TEXCOORD_0_FB_MSAA;
	snorm float2 uv1 : TEXCOORD_1_FB_MSAA;
#endif

#ifdef FOG
	float4 fogColor : FOG_COLOR;
#endif
};

ROOT_SIGNATURE
float4 main(P_in inp) : SV_TARGET
{
#ifdef BYPASS_PIXEL_SHADER
	return float4(0.0f, 0.0f, 0.0f, 0.0f);
	//return;
#else

	float2 newUV1 = float2(inp.uv1.x, inp.uv1.y);
	newUV1.y = clamp(pow(newUV1.y, 2.0), 0.2, 1.0);

#if USE_TEXEL_AA
	float4 diffuse = texture2D_AA(TEXTURE_0, TextureSampler0, inp.uv0 );
#else
	float4 diffuse = TEXTURE_0.Sample(TextureSampler0, inp.uv0);
#endif

	float4 ambient = TEXTURE_1.Sample(TextureSampler1, float2(newUV1.x, max(newUV1.y, 0.2)));
	float4 night = TEXTURE_1.Sample(TextureSampler1, float2(0.0, 1.0));
	float sun = TEXTURE_1.Sample(TextureSampler1, float2(0.0, inp.uv1.y)).g;
/*
#ifdef SEASONS_FAR
	diffuse.a = 1.0f;
#endif
*/
#if USE_ALPHA_TEST
	#ifdef ALPHA_TO_COVERAGE
		#define ALPHA_THRESHOLD 0.05
	#else
		#define ALPHA_THRESHOLD 0.5
	#endif
	if(diffuse.a < ALPHA_THRESHOLD)
		discard;
#endif
/*
#if defined(BLEND)
	diffuse.a *= inp.color.a;
#endif
*/
#if !defined(ALWAYS_LIT)
	diffuse *= ambient;
#endif

#ifndef SEASONS
	#if !USE_ALPHA_TEST && !defined(BLEND)
		diffuse.a = inp.color.a;
	#endif	

	diffuse.rgb *= inp.color.rgb;
#else
	float2 uv = inp.color.xy;
	diffuse.rgb *= lerp(1.0f, TEXTURE_2.Sample(TextureSampler2, uv).rgb*2.0f, ambient.g);
	diffuse.rgb *= inp.color.aaa;
	diffuse.a = 1.0f;
#endif

	diffuse.rgb = lerp((diffuse.rgb + (float3(0.7, 0.4, 0.1) * newUV1.x)), diffuse.rgb, sun);
	diffuse.rgb = smoothstep(0.0, 1.0, diffuse.rgb);

#ifdef FOG
	diffuse.rgb = lerp( diffuse.rgb, inp.fogColor.rgb, inp.fogColor.a );
#endif

#ifdef VR_MODE
	// On Rift, the transition from 0 brightness to the lowest 8 bit value is abrupt, so clamp to 
	// the lowest 8 bit value.
	return max(SV_TARGET, 1 / 255.0f);
#else
	return diffuse;
#endif

#endif // BYPASS_PIXEL_SHADER
}