#include "ShaderConstants.fxh"

struct V_in {
	float3 position : POSITION;
	float4 color : COLOR;
	float2 uv0 : TEXCOORD_0;
	float2 uv1 : TEXCOORD_1;
#ifdef INSTANCEDSTEREO
	uint instanceID : SV_InstanceID;
#endif
};


struct P_in {
	float4 position : SV_Position;

#ifndef BYPASS_PIXEL_SHADER
	lpfloat4 color : COLOR;
	snorm float2 uv0 : TEXCOORD_0_FB_MSAA;
	snorm float2 uv1 : TEXCOORD_1_FB_MSAA;
#endif

#ifdef FOG
	float4 fogColor : FOG_COLOR;
#endif
#ifdef GEOMETRY_INSTANCEDSTEREO
	uint instanceID : SV_InstanceID;
#endif
#ifdef VERTEXSHADER_INSTANCEDSTEREO
	uint renTarget_id : SV_RenderTargetArrayIndex;
#endif
};


static const float rA = 1.0;
static const float rB = 1.0;
static const float3 UNIT_Y = float3(0, 1, 0);
static const float DIST_DESATURATION = 56.0 / 255.0; //WARNING this value is also hardcoded in the water color, don'tchange

ROOT_SIGNATURE
P_in main(V_in inv)
{
	P_in outP;
#ifndef BYPASS_PIXEL_SHADER
	outP.uv0 = inv.uv0;
	outP.uv1 = inv.uv1;
	outP.color = inv.color;
#endif

#ifdef AS_ENTITY_RENDERER
	#ifdef INSTANCEDSTEREO
		int i = inv.instanceID;
		outP.position = mul(WORLDVIEWPROJ_STEREO[i], float4(inv.position, 1));
	#else
		outP.position = mul(WORLDVIEWPROJ, float4(inv.position, 1));
	#endif
		float3 worldPos = outP.position;
#else
		float3 worldPos = (inv.position.xyz * CHUNK_ORIGIN_AND_SCALE.w) + CHUNK_ORIGIN_AND_SCALE.xyz;
	
		// Transform to view space before projection instead of all at once to avoid floating point errors
		// Not required for entities because they are already offset by camera translation before rendering
		// World position here is calculated above and can get huge
	#ifdef INSTANCEDSTEREO
		int i = inv.instanceID;
	
		outP.position = mul(WORLDVIEW_STEREO[i], float4(worldPos, 1 ));
		outP.position = mul(PROJ_STEREO[i], outP.position);
	
	#else
		outP.position = mul(WORLDVIEW, float4( worldPos, 1 ));
		outP.position = mul(PROJ, outP.position);
	#endif

#endif
#ifdef GEOMETRY_INSTANCEDSTEREO
		outP.instanceID = inv.instanceID;
#endif 
#ifdef VERTEXSHADER_INSTANCEDSTEREO
		outP.renTarget_id = inv.instanceID;
#endif
///// find distance from the camera

#if defined(FOG) || defined(BLEND)
	#ifdef FANCY
		float3 relPos = -worldPos;
		float cameraDepth = length(relPos);
	#else
		float cameraDepth = outP.position.z;
	#endif
#endif

	///// apply fog

#ifdef FOG
	float len = cameraDepth / RENDER_DISTANCE;
#ifdef ALLOW_FADE
	len += RENDER_CHUNK_FOG_ALPHA.r;
#endif

	outP.fogColor.rgb = FOG_COLOR.rgb;
	outP.fogColor.a = clamp((len - FOG_CONTROL.x) / (FOG_CONTROL.y - FOG_CONTROL.x), 0.0, 1.0);

#endif

///// blended layer (mostly water) magic
#ifdef BLEND
	//Mega hack: only things that become opaque are allowed to have vertex-driven transparency in the Blended layer...
	//to fix this we'd need to find more space for a flag in the vertex format. color.a is the only unused part
	bool shouldBecomeOpaqueInTheDistance = inv.color.a < 0.95;
	if(shouldBecomeOpaqueInTheDistance) {
		#ifdef FANCY  /////enhance water
			float cameraDist = cameraDepth / FAR_CHUNKS_DISTANCE;
		#else
			float3 relPos = -worldPos.xyz;
			float camDist = length(relPos);
			float cameraDist = camDist / FAR_CHUNKS_DISTANCE;
		#endif //FANCY
		
		float alphaFadeOut = clamp(cameraDist, 0.0, 1.0);
		outP.color.a = lerp(inv.color.a, 1.0, alphaFadeOut);
	}
#endif

	return outP;

}
